'''io/__init__.py'''

"""
audiotsm.base.tsm.TSM objects use audiotsm.io.base.Reader
objects as input and audiotsm.io.base.Writer objects as output.

The audiotsm.io package provides Readers and Writers allowing to use
numpy arrays or wav files as input or output of a
audiotsm.base.tsm.TSM, to play the output in real-time, as well as
base classes to implement your own Readers and Writers.
"""